from collections import namedtuple
from paymentrails.exceptions.invalidFieldException import InvalidFieldException
import paymentrails.configuration


class RecipientAccountGateway(object):
    """
    PaymentRails RecipientAccount processor
    Creates and manages bank accounts
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    def findAll(self, recipient_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/'
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)

        recipientaccounts = []
        count = 0
        for recipientaccount in response['accounts']:
            recipaccount = paymentrails.recipient_account.RecipientAccount.factory(
                recipientaccount)
            newrecipientaccount = namedtuple(
                "RecipientAccount", recipaccount.keys())(*recipaccount.values())
            recipientaccounts.insert(count, newrecipientaccount)
            count = count + 1
        return recipientaccounts

    def find(self, recipient_id, recipient_account_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/' + recipient_account_id
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        recipaccount = paymentrails.recipient_account.RecipientAccount.factory(
            response)
        recipientaccount = namedtuple(
            "RecipientAccount", recipaccount.keys())(*recipaccount.values())
        return recipientaccount

    def create(self, recipient_id, body):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts'
        response = paymentrails.configuration.Configuration.client(
            self.config).post(endpoint, body)
        recipaccount = paymentrails.recipient_account.RecipientAccount.factory(
            response)
        recipientaccount = namedtuple(
            "RecipientAccount", recipaccount.keys())(*recipaccount.values())
        return recipientaccount

    def update(self, recipient_id, recipient_account_id, body):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/' + recipient_account_id
        response = paymentrails.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        if response['ok'] is True:
            return True
        return False

    def delete(self, recipient_id, recipient_account_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/accounts/' + recipient_account_id
        response = paymentrails.configuration.Configuration.client(
            self.config).delete(endpoint)
        if response['ok'] is True:
            return True
        return False



class InvalidFieldException(Exception):
    """
    An Invalid Field Exception class.
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)
    
    



class BatchSummary:
    """
    A class that facilitates Client requests to
    the PaymentRails API in regards to Batches.
    """

    _attributes = {
        "id": "",
        "amount": "",
        "completedAt": "",
        "createdAt": "",
        "currency": "",
        "description": "",
        "sentAt": "",
        "status": "",
        "totalPayments": "",
        "updatedAt": "",
        "methods": "",
    }

    @staticmethod
    def _initialize(attributes):
        fields = [
            "id",
            "amount",
            "completedAt",
            "createdAt",
            "currency",
            "description",
            "sentAt",
            "status",
            "totalPayments",
            "updatedAt",
            "methods",
        ]

        for field in fields:
            if attributes.get('batchSummary') is not None:
                BatchSummary._attributes[field] = attributes['batchSummary'].get(field)

        return BatchSummary._attributes

    @staticmethod
    def factory(attributes):
        instance = BatchSummary._initialize(attributes)
        return instance



from collections import namedtuple
from paymentrails.exceptions.invalidFieldException import InvalidFieldException
import paymentrails.configuration


class BalancesGateway(object):
    """
    PaymentRails Balance module
    Manages balances
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    def find(self, term=""):
        if term is None:
            raise InvalidFieldException("Term cannot be None")
        endpoint = '/v1/profile/balances/' + term
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        oldbalance = paymentrails.balances.Balances.factory(response)
        balance = namedtuple("Balances", oldbalance.keys())(*oldbalance.values())
        return balance



from paymentrails.configuration import Configuration


class RecipientAccount:
    """
    A class that facilitates Client requests to
    the PaymentRails API in regards to Recipient Accounts.
    """

    _attributes = {
        "id": "",
        "primary": "",
        "currency": "",
        "recipientAccountId": "",
        "routeType": "",
        "recipientFees": "",
        "emailAddress": "",
        "country": "",
        "type": "",
        "iban": "",
        "accountNum": "",
        "accountHolderName": "",
        "swiftBic": "",
        "branchId": "",
        "bankId": "",
        "bankName": "",
        "bankAddress": "",
        "bankCity": "",
        "bankRegionCode": "",
        "bankPostalCode": ""
    }

    @staticmethod
    def findAll(recipient_id):
        """
        Retrieve all the recipient accounts
            A recipient_id is required::
            RecipientAccount.findAll('R-fjeracjmuflh')
        """
        return Configuration.gateway().recipient_account.findAll(recipient_id)

    @staticmethod
    def find(recipient_id, recipient_account_id):
        """
        Retrieve a recipient account
            A recipient_id and recipient_account_id are required::
            RecipientAccount.find('R-fjeracjmuflh','A-2DQMpN4jurTFn9gRxobx4C')
        """
        return Configuration.gateway().recipient_account.find(recipient_id, recipient_account_id)
       
    @staticmethod
    def create(recipient_id, body):
        """
        Create a recipient account
            A recipient_id and body are required::
            RecipientAccount.create('R-4625iLug2GKqKZG2WzAf3e','payload')
        """
        return Configuration.gateway().recipient_account.create(recipient_id, body)

    @staticmethod
    def update(recipient_id, recipient_account_id, body):
        """
        Update a recipient account
            A recipient_id, recipient_account_id, and body are required::
            RecipientAccount.update('R-fjeracjmuflh','A-2DQMpN4jurTFn9gRxobx4C',
            {"accountHolderName": "Acer Philips"})
        """
        return Configuration.gateway().recipient_account.update(recipient_id,
                                                                recipient_account_id, body)

    @staticmethod
    def delete(recipient_id, recipient_account_id):
        """
        Delete a recipient account
            A recipient_id and recipient_account_id are required::
            RecipientAccount.delete('R-fjeracjmuflh','A-2DQMpN4jurTFn9gRxobx4C')
        """
        return Configuration.gateway().recipient_account.delete(recipient_id, recipient_account_id)

    @staticmethod
    def _initialize(attributes):
        fields = [
            "id",
            "primary",
            "currency",
            "recipientAccountId",
            "routeType",
            "recipientFees",
            "emailAddress",
            "country",
            "type",
            "iban",
            "accountNum",
            "accountHolderName",
            "swiftBic",
            "branchId",
            "bankId",
            "bankName",
            "bankAddress",
            "bankCity",
            "bankRegionCode",
            "bankPostalCode",
        ]

        for field in fields:
            if attributes.get('account') is None:
                RecipientAccount._attributes[field] = attributes.get(field)
            elif attributes.get('account') is not None:
                RecipientAccount._attributes[field] = attributes['account'].get(field)

        return RecipientAccount._attributes

    @staticmethod
    def factory(attributes):
        instance = RecipientAccount._initialize(attributes)
        return instance



from collections import namedtuple
from paymentrails.exceptions.invalidFieldException import InvalidFieldException
import paymentrails.configuration



class BatchGateway(object):
    """
    PaymentRails Batch processor
    Creates and manages batches
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    def find(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        tempbatch = paymentrails.batch.Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch

    def create(self, body):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/'
        response = paymentrails.configuration.Configuration.client(
            self.config).post(endpoint, body)
        tempbatch = paymentrails.batch.Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch

    def update(self, batchid, body):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/' + batchid
        response = paymentrails.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        if response['ok'] is True:
            return True
        return False

    def delete(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid
        response = paymentrails.configuration.Configuration.client(
            self.config).delete(endpoint)
        if response['ok'] is True:
            return True
        return False

    def search(self, page=1, pagenumber=10, term=""):
        endpoint = '/v1/batches?search=' + term + \
            '&page=' + str(page) + '&pageSize=' + str(pagenumber)
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        batches = []
        count = 0
        for batch in response['batches']:
            tempbatch = paymentrails.batch.Batch.factory(batch)
            newbatch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
            batches.insert(count, newbatch)
            count = count + 1
        return batches

    def summary(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid + '/summary'
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        tempbatchsummary = paymentrails.batch_summary.BatchSummary.factory(
            response)
        batchsummary = namedtuple("BatchSummary", tempbatchsummary.keys())(
            *tempbatchsummary.values())
        return batchsummary

    def generate_quote(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid + '/generate-quote'
        response = paymentrails.configuration.Configuration.client(
            self.config).post(endpoint, {})
        tempbatch = paymentrails.batch.Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch

    def process_batch(self, batchid):
        if batchid is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batchid + '/start-processing'
        response = paymentrails.configuration.Configuration.client(
            self.config).post(endpoint, {})
        tempbatch = paymentrails.batch.Batch.factory(response)
        batch = namedtuple("Batch", tempbatch.keys())(*tempbatch.values())
        return batch




from paymentrails.client import Client
# import paymentrails.client
from paymentrails.gateway import Gateway

class Configuration(object):
    """
    Configuration registry
    """

    public_key = ''
    private_key = ''
    enviroment = 'https://api.paymentrails.com'

    @staticmethod
    def gateway():
        return Gateway(config=Configuration.instantiate())

    @staticmethod
    def client(config):
        return Client.create(config)

    @staticmethod
    def instantiate():
        return Configuration()

    @staticmethod
    def set_public_key(public_key):
        """
        Set method for the public key
        """
        Configuration.public_key = public_key

    @staticmethod
    def get_public_key():
        """
        Get method for the public key
        """
        return Configuration.public_key

    @staticmethod
    def get_private_key():
        """
        Get method for the private key
        """
        return Configuration.private_key

    @staticmethod
    def set_private_key(private_key):
        """
        Get method for the private key
        """
        Configuration.private_key = private_key

    @staticmethod
    def set_enviroment(enviroment):
        """
        Set method to change the enviroment
        """
        if enviroment == 'production':
            Configuration.enviroment = 'https://api.paymentrails.com'
        elif enviroment == 'development':
            Configuration.enviroment = 'http://api.railz.io'
        elif enviroment == 'integration':
            Configuration.enviroment = 'http://api.local.dev:3000'



from paymentrails.configuration import Configuration

class Recipient:
    """
    A class that facilitates Client requests to
    the PaymentRails API in regards to Recipients.
    """

    _attributes = {
        "id": "",
        "routeType": "",
        "estimatedFees": "",
        "referenceId": "",
        "email": "",
        "name": "",
        "lastName": "",
        "firstName": "",
        "type": "",
        "taxType": "",
        "status": "",
        "language": "",
        "complianceStatus": "",
        "dob": "",
        "passport": "",
        "updatedAt": "",
        "createdAt": "",
        "gravatarUrl": "",
        "governmentId": "",
        "ssn": "",
        "primaryCurrency": "",
        "merchantId": "",
        "payoutMethod": "",
        "compliance": "",
        "accounts": "",
        "address": "",
    }


    @staticmethod
    def find(recipient_id, term=""):
        """
        Retrieve a recipient
            A recipient_id is required::
            Recipient.find('R-fjeracjmuflh')
        """
        recip = Configuration.gateway().recipient.find(recipient_id, term)
        return recip
    @staticmethod
    def create(body):
        """
        Create a recipient
            A body is required::
            Recipient.create({"type": "individual", "firstName": "John",
                                    "lastName": "Smith", "email": "jh@edxample.com"})
        """
        return Configuration.gateway().recipient.create(body)

    @staticmethod
    def update(recipient_id, body):
        """
        Update a recipient
            A recipient_id and body are required::
            Recipient.update({'firstName': 'tom'})
        """
        return Configuration.gateway().recipient.update(recipient_id, body)

    @staticmethod
    def delete(recipient_id):
        """
        Delete a recipient
            A recipient_id is required::
            Recipient.delete('R-fjeracjmuflh')
        """
        return Configuration.gateway().recipient.delete(recipient_id)

    @staticmethod
    def search(page=1, page_number=10, term=""):
        """
        Query for a recipient
            Recipient.search(1,10,'test')
        """
        return Configuration.gateway().recipient.search(page, page_number, term)

    @staticmethod
    def _initialize(attributes):
        """Initialize fields and return a dict of attributes."""

        fields = [
            "id",
            "routeType",
            "estimatedFees",
            "id",
            "referenceId",
            "email",
            "name",
            "lastName",
            "firstName",
            "type",
            "taxType",
            "status",
            "language",
            "complianceStatus",
            "dob",
            "passport",
            "updatedAt",
            "createdAt",
            "gravatarUrl",
            "governmentId",
            "ssn",
            "primaryCurrency",
            "merchantId",
            "payoutMethod",
            "compliance",       # TODO: Factory
            "accounts",
            "address",          #TODO: Factory
        ]

        for field in fields:
            if attributes.get('recipient') is None:
                Recipient._attributes[field] = attributes.get(field)
            elif attributes['recipient'].get(field) is not None:
                Recipient._attributes[field] = attributes['recipient'][field]

        return Recipient._attributes

    @staticmethod
    def factory(attributes):
        """Creates an instance of Recipient and returns it. """
        instance = Recipient._initialize(attributes)
        return instance




import time
import hmac
import hashlib
import json
import requests

import paymentrails.exceptions.invalidServerConnectionException
import paymentrails.exceptions.unexpectedException
import paymentrails.exceptions.notFoundException
import paymentrails.exceptions.authenticationException
import paymentrails.exceptions.authorizationException
import paymentrails.exceptions.invalidFieldException
import paymentrails.exceptions.tooManyRequestsException
import paymentrails.exceptions.downForMaintenanceException
import paymentrails.exceptions.malformedException
import paymentrails.exceptions.invalidStatusException


class Client(object):
    """
    A class that makes Client requests to the API.
    """

    def __init__(self, config):
        self.config = config

    @staticmethod
    def create(config):
        """
        Factory create method
        """
        return Client(config)

    def get(self, endpoint):
        """
        Makes an HTTP GET request to the API
        """
        try:
            timestamp = int(time.time())
            authorization = self.generate_authorization(
                timestamp, "GET", endpoint, self.config)

            headers = {'Content-Type': 'application/json',
                       'Authorization': authorization,
                       'X-PR-Timestamp': str(timestamp)}

            request = requests.get(
                self.config.enviroment + endpoint, headers=headers)
            if request.status_code != 200 and request.status_code != 204:
                self.throw_status_code_exception(
                    request.status_code, request.content.decode("utf-8"))

            data = json.loads(request.content.decode("utf-8"))
            return data

        except requests.exceptions.RequestException:
            raise paymentrails.exceptions.invalidServerConnectionException.InvalidServerConnectionException(
                "Invalid Connection to the server")

    def post(self, endpoint, body):
        """
        Makes an HTTP POST request to the API
        """
        try:
            timestamp = int(time.time())
            authorization = self.generate_authorization(
                timestamp, "POST", endpoint, self.config, body)

            headers = {'Content-Type': 'application/json',
                       'Authorization': authorization,
                       'X-PR-Timestamp': str(timestamp)}

            request = requests.post(self.config.enviroment +
                                    endpoint, headers=headers, json=body)
            if request.status_code != 200 and request.status_code != 204:
                self.throw_status_code_exception(
                    request.status_code, request.content.decode("utf-8"))
            data = json.loads(request.content.decode("utf-8"))
            return data

        except requests.exceptions.RequestException:
            raise paymentrails.exceptions.invalidServerConnectionException.InvalidServerConnectionException(
                "Invalid Connection to the server")

    def patch(self, endpoint, body):
        """
        Makes an HTTP PATCH request to the API
        """
        try:

            timestamp = int(time.time())

            authorization = self.generate_authorization(
                timestamp, "PATCH", endpoint, self.config, body)

            headers = {'Content-Type': 'application/json',
                       'Authorization': authorization,
                       'X-PR-Timestamp': str(timestamp)}

            request = requests.patch(self.config.enviroment + endpoint,
                                     headers=headers, json=body)
            if request.status_code != 200 and request.status_code != 204:
                self.throw_status_code_exception(
                    request.status_code, request.content.decode("utf-8"))
            data = json.loads(request.content.decode("utf-8"))
            return data

        except requests.exceptions.RequestException:
            raise paymentrails.exceptions.invalidServerConnectionException.InvalidServerConnectionException(
                "Invalid Connection to the server")

    def delete(self, endpoint):
        """
        Makes an HTTP DELETE request to the API
        """
        try:

            timestamp = int(time.time())

            authorization = self.generate_authorization(
                timestamp, "DELETE", endpoint, self.config)

            headers = {'Content-Type': 'application/json',
                       'Authorization': authorization,
                       'X-PR-Timestamp': str(timestamp)}
            request = requests.delete(
                self.config.enviroment + endpoint, headers=headers)
            if request.status_code != 200 and request.status_code != 204:
                self.throw_status_code_exception(
                    request.status_code, request.content.decode("utf-8"))

            data = json.loads(request.content.decode("utf-8"))
            return data
        except requests.exceptions.RequestException:
            raise paymentrails.exceptions.invalidServerConnectionException.InvalidServerConnectionException(
                "Invalid Connection to the server")

    @staticmethod
    def generate_authorization(timestamp, method, endpoint, config, body=''):
        """
        Generates an authorization signature for the request header
        """
        if body != '':
            body = json.dumps(body)
        message = str(timestamp) + '\n' + method + \
            '\n' + endpoint + '\n' + body + '\n'
        key = bytes(config.get_private_key().encode('utf-8'))
        signature = hmac.new(key, msg=message.encode(
            'utf-8'), digestmod=hashlib.sha256).hexdigest()
        return 'prsign ' + config.get_public_key() + ':' + signature

    @staticmethod
    def throw_status_code_exception(status_code, message):
        """
        Throws an exception based on the type of error
        """
        if status_code == 400:
            raise paymentrails.exceptions.malformedException.MalformedException(
                message)
        elif status_code == 401:
            raise paymentrails.exceptions.authenticationException.AuthenticationException(
                message)
        elif status_code == 403:
            raise paymentrails.exceptions.authorizationException.AuthorizationException(
                message)
        elif status_code == 404:
            raise paymentrails.exceptions.notFoundException.NotFoundException(
                message)
        elif status_code == 406:
            raise paymentrails.exceptions.invalidStatusException.InvalidStatusException(
                message)
        elif status_code == 429:
            raise paymentrails.exceptions.tooManyRequestsException.TooManyRequestsException(
                message)
        elif status_code == 500:
            raise paymentrails.exceptions.invalidServerConnectionException.InvalidServerConnectionException(
                message)
        elif status_code == 503:
            raise paymentrails.exceptions.downForMaintenanceException.DownForMaintenanceException(
                message)
        else:
            raise paymentrails.exceptions.unexpectedException.UnexpectedException(
                'Unexpected HTTP_RESPONSE # ' + str(status_code))



from paymentrails.configuration import Configuration


class Batch:
    """
    A class that facilitates Client requests to
    the PaymentRails API in regards to Batches.
    """

    _attributes = {
        "id": "",
        "amount": "",
        "completedAt": "",
        "createdAt": "",
        "currency": "",
        "description": "",
        "sentAt": "",
        "status": "",
        "totalPayments": "",
        "updatedAt": "",
    }

    @staticmethod
    def find(batch_id):
        """
        Retrieve a batch
            A batch_id is required::
            Batch.find('B-fjeracjmuflh')
        """
        return Configuration.gateway().batch.find(batch_id)

    @staticmethod
    def create(body):
        """
        Create a batch
            A body is required::
            Batch.create(
                {"payments":[{"recipient":{"id":"R-SBAHDK3DK6M7SUEM"},
                "sourceAmount":"65","memo":"","sourceCurrency":"CAD"}]})
        """
        return Configuration.gateway().batch.create(body)

    @staticmethod
    def update(batch_id, body):
        """
        Update a batch
            A batch_id and body are required::
            Batch.update('B-fjeracjmuflh',{"payments":[{"recipient":{"id":"R-3DF7FAF680739541",
            "email":"jsmith@example.com"},"sourceAmount":65,"memo":"Salary",
            "sourceCurrency":"CAD"}]})
        """
        return Configuration.gateway().batch.update(batch_id, body)

    @staticmethod
    def delete(batch_id):
        """
        Delete a batch
            A batch_id is required::
            Batch.delete('B-fjeracjmuflh')
        """
        return Configuration.gateway().batch.delete(batch_id)

    @staticmethod
    def search(page=1, page_number=10, term=""):
        """
        Query for a batch
            Batch.search(1,10,'test')
        """
        return Configuration.gateway().batch.search(page, page_number, term)

    @staticmethod
    def summary(batch_id):
        """
        Retrieve a summary about a specific batch
            A batch_id is required::
            Batch.summary('B-fjeracjmuflh')
        """
        return Configuration.gateway().batch.summary(batch_id)

    @staticmethod
    def generate_quote(batch_id):
        """
        Generate a quote for a batch
            A batch_id is required::
            Batch.generate_quote('B-fjeracjmuflh')
        """
        return Configuration.gateway().batch.generate_quote(batch_id)

    @staticmethod
    def process_batch(batch_id):
        """
        Process a batch
            A batch_id is required::
            Batch.process_batch('B-fjeracjmuflh')
        """
        return Configuration.gateway().batch.process_batch(batch_id)

    @staticmethod
    def _initialize(attributes):
        """Initialize fields and return a dict of attributes."""
        fields = [
            "id",
            "amount",
            "completedAt",
            "createdAt",
            "currency",
            "description",
            "sentAt",
            "status",
            "totalPayments",
            "updatedAt",
        ]

        for field in fields:
            if attributes.get('batch') is None:
                Batch._attributes[field] = attributes.get(field)
            elif attributes['batch'].get(field) is not None:
                Batch._attributes[field] = attributes['batch'][field]

        return Batch._attributes

    @staticmethod
    def factory(attributes):
        """Creates an instance of Batch and returns it. """
        instance = Batch._initialize(attributes)
        return instance



from paymentrails.configuration import Configuration
from paymentrails.recipient import Recipient
from paymentrails.balances import Balances
from paymentrails.client import Client
from paymentrails.batch import Batch
from paymentrails.payment import Payment
from paymentrails.recipient_account import RecipientAccount
from paymentrails.gateway import Gateway
from paymentrails.batch_summary import BatchSummary



from paymentrails.configuration import Configuration


class Balances:
    """
    A class that facilitates Client requests to
    the PaymentRails API in regards to Balances.
    """
    _attributes = {}


    @staticmethod
    def find(term=""):
        """
        Retrieve a balance
            Balances.find()
        """
        return Configuration.gateway().balances.find(term)
    @staticmethod
    def _initialize(attributes):
        """Initialize fields and return a dict of attributes."""
        for field in attributes.get('balances'):
            Balances._attributes.update({field:""})
            Balances._attributes[field] = attributes['balances'].get(field)

        return Balances._attributes

    @staticmethod
    def factory(attributes):
        """Creates an instance of Balances and returns it. """
        instance = Balances._initialize(attributes)
        return instance



from paymentrails.configuration import Configuration


class Payment:
    """
    A class that facilitates Client requests to
    the PaymentRails API in regards to Payments.
    """

    _attributes = {
        'id': "",
        'status': "",
        'isSupplyPayment': "",
        'returnedAmount': "",
        'sourceAmount': "",
        'sourceCurrency': "",
        'targetAmount': "",
        'targetCurrency': "",
        'exchangeRate': "",
        'fees': "",
        'recipientFees': "",
        'fxRate': "",
        'memo': "",
        'externalId': "",
        'processedAt': "",
        'createdAt': "",
        'updatedAt': "",
        'merchantFees': "",
        'compliance': "",
        'payoutMethod': "",
    }

    @staticmethod
    def find(payment_id, batch_id):
        """
        Retrieve a payment
            A payment_id and batch_id are required::
            Payment.find('P-dejrtewdsj',B-fjeracjmuflh')
        """
        return Configuration.gateway().payment.find(payment_id, batch_id)

    @staticmethod
    def create(body, batch_id):
        """
        Create a payment
            A body and batch_id are required::
            Payment.create(
               {"recipient":{"id":"R-91XNJBKM30F06"},"sourceAmount":"100.10",
               "memo":"Freelance payment"}
        """
        return Configuration.gateway().payment.create(body, batch_id)

    @staticmethod
    def update(payment_id, body, batch_id):
        """
        Update a payment
            A payment_id, batch_id, and body are required::
            Payment.update('B-fjeracjmuflh',{"sourceAmount":"900.90"},'P-jddfjwojd')
        """
        return Configuration.gateway().payment.update(payment_id, body, batch_id)

    @staticmethod
    def delete(payment_id, batch_id):
        """
        Delete a payment
            A payment_id and batch_id are required::
            Payment.delete('P-dejrtewdsj',B-fjeracjmuflh')
        """
        return Configuration.gateway().payment.delete(payment_id, batch_id)

    @staticmethod
    def search(page=1, page_number=10, term=""):
        """
        Query for a payment
            Payment.search(1,10,'test')
        """
        return Configuration.gateway().payment.search(page, page_number, term)

    @staticmethod
    def _initialize(attributes):
        """Initialize fields and return a dict of attributes."""

        fields = [
            'id',
            'status',
            'isSupplyPayment',
            'returnedAmount',
            'sourceAmount',
            'sourceCurrency',
            'targetAmount',
            'targetCurrency',
            'exchangeRate',
            'fees',
            'recipientFees',
            'fxRate',
            'memo',
            'externalId',
            'processedAt',
            'createdAt',
            'updatedAt',
            'merchantFees',
            'compliance',
            'payoutMethod',
        ]

        for field in fields:
            if attributes.get('payment') is None:
                Payment._attributes[field] = attributes.get(field)
            elif attributes['payment'].get(field) is not None:
                Payment._attributes[field] = attributes['payment'][field]

        return Payment._attributes

    @staticmethod
    def factory(attributes):
        """Creates an instance of Payment and returns it. """
        instance = Payment._initialize(attributes)
        return instance



from collections import namedtuple
from paymentrails.exceptions.invalidFieldException import InvalidFieldException
import paymentrails.configuration

class RecipientGateway(object):
    """
    PaymentRails Recipient processor
    Creates and manages transactions
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    def find(self, recipient_id, term):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id + '/' + term
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        recip = paymentrails.recipient.Recipient.factory(response)
        recipient = namedtuple("Recipient", recip.keys())(*recip.values())
        count = 0
        for account in recipient.accounts:
            recipient.accounts[count] = namedtuple(
                "RecipientAccount", account.keys())(*account.values())
            count = count + 1
        return recipient

    def create(self, body):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/recipients/'
        response = paymentrails.configuration.Configuration.client(
            self.config).post(endpoint, body)
        recip = paymentrails.recipient.Recipient.factory(response)
        recipient = namedtuple("Recipient", recip.keys())(*recip.values())
        return recipient

    def update(self, recipient_id, body):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None")
        endpoint = '/v1/recipients/' + recipient_id
        response = paymentrails.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        if response['ok'] is True:
            return True
        return False

    def delete(self, recipient_id):
        if recipient_id is None:
            raise InvalidFieldException("Recipient id cannot be None.")
        endpoint = '/v1/recipients/' + recipient_id
        response = paymentrails.configuration.Configuration.client(
            self.config).delete(endpoint)
        if response['ok'] is True:
            return True
        return False

    def search(self, page, page_number, term):
        endpoint = '/v1/recipients?search=' + term + '&page=' + \
            str(page) + '&pageSize=' + str(page_number)
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        recipients = []
        count = 0
        for recipient in response['recipients']:
            temp = paymentrails.recipient.Recipient.factory(recipient)

            recipient = namedtuple("Recipient", temp.keys())(*temp.values())
            recipients.insert(count, recipient)

            count = count + 1
        return recipients



from setuptools import setup

setup (
    name='paymentrails',
    version='0.1',
    packages=["paymentrails", "paymentrails.exceptions"], 
    package_data={"paymentrails": ["ssl/*"]},
    install_requires=['requests==2.13.0'],
    author='Jesse',
    author_email='jesse.silber@paymentrails.com',
    summary='Payment Rails Python SDK',
    url='',
    license='',
    long_description='A native Python SDK for the Payment Rails API',
)



from collections import namedtuple
from paymentrails.exceptions.invalidFieldException import InvalidFieldException
import paymentrails.configuration


class PaymentGateway(object):
    """
    PaymentRails Payment processor
    Creates and manages transactions
    """

    def __init__(self, gateway, config):
        self.gateway = gateway
        self.config = config

    def find(self, payment_id, batch_id):
        if payment_id is None:
            raise InvalidFieldException("Payment id cannot be None.")
        if batch_id is None:
            raise InvalidFieldException("Batch id cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/' + payment_id
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        temppayment = paymentrails.payment.Payment.factory(response)
        payment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
        return payment

    def create(self, body, batch_id):
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/'
        response = paymentrails.configuration.Configuration.client(
            self.config).post(endpoint, body)
        temppayment = paymentrails.payment.Payment.factory(response)
        payment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
        return payment

    def update(self, payment_id, body, batch_id):
        if payment_id is None:
            raise InvalidFieldException("Payment id cannot be None.")
        if body is None:
            raise InvalidFieldException("Body cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/' + payment_id
        response = paymentrails.configuration.Configuration.client(
            self.config).patch(endpoint, body)
        if response['ok'] is True:
            return True
        return False

    def delete(self, payment_id, batch_id):
        if payment_id is None:
            raise InvalidFieldException("Payment id cannot be None.")
        endpoint = '/v1/batches/' + batch_id + '/payments/' + payment_id
        response = paymentrails.configuration.Configuration.client(
            self.config).delete(endpoint)
        if response['ok'] is True:
            return True
        return False

    def search(self, page=1, page_number=10, term=""):
        endpoint = '/v1/batches?search=' + term + \
            '&page=' + str(page) + '&pageSize=' + str(page_number)
        response = paymentrails.configuration.Configuration.client(
            self.config).get(endpoint)
        payments = []
        count = 0
        for payment in response['batches']:
            temppayment = paymentrails.payment.Payment.factory(payment)
            newpayment = namedtuple("Payment", temppayment.keys())(*temppayment.values())
            payments.insert(count, newpayment)
            count = count + 1
        return payments



from paymentrails.recipient_gateway import RecipientGateway
from paymentrails.balances_gateway import BalancesGateway
from paymentrails.batch_gateway import BatchGateway
from paymentrails.payment_gateway import PaymentGateway
from paymentrails.recipient_account_gateway import RecipientAccountGateway
import paymentrails.configuration


class Gateway(object):
    """
    PaymentRails gateway module
    """

    def __init__(self, config=None, **kwargs):
        if isinstance(config, paymentrails.configuration.Configuration):
            self.config = config
        else:
            self.config = paymentrails.configuration.Configuration(
                public_key=kwargs.get("public_key"),
                private_key=kwargs.get("private_key")
            )
        self.recipient = RecipientGateway(self, config)
        self.balances = BalancesGateway(self, config)
        self.batch = BatchGateway(self, config)
        self.payment = PaymentGateway(self, config)
        self.recipient_account = RecipientAccountGateway(self, config)
